/*
 *
 *  KBluetooth4 - KDE Bluetooth Framework
 *
 *  Copyright (C) 2008  Tom Patzig <tpatzig@suse.de>
 *
 *  This file is part of kbluetooth4.
 *
 *  kbluetooth4 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  kbluetooth4 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with kbluetooth4; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
*/


#ifndef KBLUETRAY_H
#define KBLUETRAY_H

#include <QSystemTrayIcon>

#include <kdialog.h>
#include <kaboutapplicationdialog.h>
#include <kaboutdata.h>
#include <klocalizedstring.h>
#include <KConfig>
#include <KConfigGroup>
#include <KFileDialog>
#include <KIcon>
#include <KProcess>
#include <QtGui>
#include <QtDBus>

#include <QMainWindow>


#include <solid/control/bluetoothmanager.h>
#include <solid/control/bluetoothsecurity.h>


#include "adapterconfig.h"
#include "authagent.h"
#include "passkeyagent.h"
#include "inputwizard/wizarddialog.h"
#include "device-manager/devicemanager.h"
#include "obexsession/obexsession.h"
#include "deviceselector/devicesel.h"


class KBlueTray : public QObject
{
Q_OBJECT

public:
	 KBlueTray(QObject* parent = 0);
	 ~KBlueTray();

private:

	 QStringList modes;
	 QString toolTipInfo;
	 QSystemTrayIcon* tray;
//	 KSystemTrayIcon* tray;
	 QMenu* mainmenu;

	 QAction* quitAction;
	 QAction* aboutAction;
	 QAction* adapterAction;
	 QAction* wizardAction;
	 QAction* sendToAction;
	 QAction* lockConfigureAction;
	 QAction* lockEnableAction;
	 QMenu* kbluelockMenu;
	 
	 KConfig* config;
	 KConfigGroup* confGroup;

	 AdapterConfig* adapterConfig;

	 Solid::Control::BluetoothSecurity* secure;
	 Solid::Control::BluetoothInterface* adapter;

	 AuthAgent* authagent;
	 PasskeyAgent* passagent;

	 ObexSession* session;

	 QString defaultAdapterUBI;
	 bool online;

	 DeviceSel* devSelector;
	 DeviceSel* lockSelector;
	 QString fileToSend;
	 QProgressDialog* progress;
	
	 qulonglong totalFileSize;
	 QString lockDevice;
	 bool kblueLockEnabled;

	 QDBusConnection* dbus;




public slots:
	 void slotQuitApp(bool);
	 void showAboutInfo();
	 void showAdapterInterface();
	 void showWizard();
	 void showManager();
	 void sendFile();
	 void slotSendFile(const QString&,const QString&);

	 void obexSessionReady();
	 void onlineMode(Solid::Control::BluetoothManager&);
	 void offlineMode();

	 void adapterAdded(const QString&);
	 void adapterRemoved(const QString&);
	 void defaultAdapterChanged(const QString&);
	 void slotFileSendFinished();
	 void slotTransferProgress(qulonglong);
	 void slotFileSendCanceled();
	 void slotFileSendStarted(const QString& ,const QString& ,qulonglong );

	 void enableLock();
	 void enableBlueLock(QString,const QString&);
	 void slotValidateLockDev(const QString&);
	 void slotConfigureKBlueLock();
	 void disableBlueLock();
	 void slotBlueSelectorCancelled();
	 void slotTargetDeviceFound(const QString&,int,int);
	 void slotModeChanged(Solid::Control::BluetoothInterface::Mode mode);
	 void slotNameChanged(const QString& name);
	 void slotRemoteDeviceConnected(const QString&);
	 void slotRemoteDeviceDisconnected(const QString&);


};


#endif
